module Dapp
  module Kube
    module Kubernetes::Client::Resource
      class Pod < Base
        # Returns:
        #   nil: no such condition yet
        #   "True" string: ready
        #   "False" string: not ready
        def ready_condition_status
          rd = self.ready_condition
          return nil unless rd
          return rd['status']
        end

        def ready_condition
          status.fetch('conditions', {}).find {|condition| condition['type'] == 'Ready'}
        end

        def container_id(container_name)
          container_status = spec.fetch('status', {})
            .fetch('containerStatuses')
            .find {|cs| cs['name'] == container_name}

          if container_status
            container_status['containerID']
          else
            nil
          end
        end

        def container_state(container_name)
          container_status = status
            .fetch('containerStatuses', [])
            .find {|cs| cs['name'] == container_name}

          if container_status
            container_state, container_state_data = container_status.fetch('state', {}).first
            [container_state, container_state_data]
          else
            [nil, {}]
          end
        end

        def phase
          status.fetch('phase', nil)
        end

        def containers_names
          spec.fetch('spec', {})
            .fetch('containers', [])
            .map {|container_spec| container_spec['name']}
        end

        def restart_policy
          spec
            .fetch('spec', {})
            .fetch('restartPolicy', nil)
        end
      end # Pod
    end # Kubernetes::Client::Resource
  end # Kube
end # Dapp
